##############################################################################
#
# Copyright (c) 2018 Nexedi SA and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import os
import urlparse
import multiprocessing
from BaseHTTPServer import HTTPServer
from BaseHTTPServer import BaseHTTPRequestHandler

from utils import SlapOSInstanceTestCase
from utils import findFreeTCPPort

from selenium import webdriver
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities


# for development: debugging logs and install Ctrl+C handler
if os.environ.get('DEBUG'):
  import logging
  logging.basicConfig(level=logging.DEBUG)
  import unittest
  unittest.installHandler()


class TestRemoteDriver(SlapOSInstanceTestCase):
  """Test the remote driver on a minimal web server.
  """
  @classmethod
  def getSoftwareURLList(cls):
    return (os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'software.cfg')), )

  def setUp(self):
    """Start a minimal web server.
    """
    class TestHandler(BaseHTTPRequestHandler):
      def do_GET(self):
        self.send_response(200)
        self.end_headers()
        self.wfile.write('''
          <html>
            <title>Test page</title>
          </html>''')

    ip = os.environ.get('LOCAL_IPV4', '127.0.1.1')
    port = findFreeTCPPort(ip)
    server = HTTPServer((ip, port), TestHandler)
    self.server_process = multiprocessing.Process(target=server.serve_forever)
    self.server_process.start()
    self.server_url = 'http://%s:%s/' % (ip, port)

  def tearDown(self):
    self.server_process.terminate()
    self.server_process.join()

  def test_firefox(self):
    parameter_dict = self.computer_partition.getConnectionParameterDict()
    webdriver_url = parameter_dict['url']

    driver = webdriver.Remote(
         command_executor=webdriver_url,
         desired_capabilities=DesiredCapabilities.FIREFOX)

    driver.get(self.server_url)
    self.assertEqual('Test page', driver.title)
    driver.close()

  def test_firefox_desired_version(self):
    parameter_dict = self.computer_partition.getConnectionParameterDict()
    webdriver_url = parameter_dict['url']

    desired_capabilities = DesiredCapabilities.FIREFOX.copy()
    desired_capabilities['version'] = '51.0.1'
    driver = webdriver.Remote(
         command_executor=webdriver_url,
         desired_capabilities=desired_capabilities)
    self.assertIn(
        'Gecko/20100101 Firefox/51.0',
        driver.execute_script('return navigator.userAgent'))
    driver.close()

    desired_capabilities['version'] = '52.9.0esr'
    driver = webdriver.Remote(
         command_executor=webdriver_url,
         desired_capabilities=desired_capabilities)
    self.assertIn(
        'Gecko/20100101 Firefox/52.0',
        driver.execute_script('return navigator.userAgent'))
    driver.close()

  def test_parralel_connections(self):
    parameter_dict = self.computer_partition.getConnectionParameterDict()
    webdriver_url = parameter_dict['url']

    queue = multiprocessing.Queue()
    def _test(q):
      driver = webdriver.Remote(
           command_executor=webdriver_url,
           desired_capabilities=DesiredCapabilities.FIREFOX)

      driver.get(self.server_url)
      q.put(driver.title == 'Test page')
      driver.close()

    nb_workers = 10
    workers = []
    for i in range(nb_workers):
      worker = multiprocessing.Process(target=_test, args=(queue, ))
      worker.start()
      workers.append(worker)

    _ = [worker.join() for worker in workers]

    self.assertEqual(
        [True] * nb_workers,
        [queue.get() for _ in range(nb_workers)])
